@@ -236,8 +236,8 @@ boxed_new(JSContext   *context,
                 void *gboxed;
 
                 rval = JSVAL_NULL;
-                gjs_invoke_c_function(context, func_info, obj,
-                                      0, NULL, &rval);
+                gjs_invoke_c_function_uncached(context, func_info, obj,
+                                               0, NULL, &rval);
 
                 g_base_info_unref((GIBaseInfo*) func_info);
 
@@ -448,8 +448,8 @@ boxed_constructor(JSContext *context,
      * class as us, but if we're constructing an instance, the prototype
      * has the same class.
      */
-    obj_class = JS_GetClass(context, obj);
-    proto_class = JS_GetClass(context, proto);
+    obj_class = JS_GET_CLASS(context, obj);
+    proto_class = JS_GET_CLASS(context, proto);
 
     is_proto = (obj_class != proto_class);
 
@@ -1014,7 +1014,7 @@ gjs_lookup_boxed_class(JSContext    *context,
 
     prototype = gjs_lookup_boxed_prototype(context, info);
 
-    return JS_GetClass(context, prototype);
+    return JS_GET_CLASS(context, prototype);
 }
 
 /* Check if the type of the boxed is "simple" - every field is a non-pointer
@@ -1043,19 +1043,10 @@ struct_is_simple(GIStructInfo *info)
             case GI_TYPE_TAG_UINT16:
             case GI_TYPE_TAG_INT32:
             case GI_TYPE_TAG_UINT32:
-            case GI_TYPE_TAG_SHORT:
-            case GI_TYPE_TAG_USHORT:
-            case GI_TYPE_TAG_INT:
-            case GI_TYPE_TAG_UINT:
             case GI_TYPE_TAG_INT64:
             case GI_TYPE_TAG_UINT64:
-            case GI_TYPE_TAG_LONG:
-            case GI_TYPE_TAG_ULONG:
-            case GI_TYPE_TAG_SSIZE:
-            case GI_TYPE_TAG_SIZE:
             case GI_TYPE_TAG_FLOAT:
             case GI_TYPE_TAG_DOUBLE:
-            case GI_TYPE_TAG_TIME_T:
                 break;
             case GI_TYPE_TAG_VOID:
             case GI_TYPE_TAG_GTYPE:
@@ -1068,43 +1059,43 @@ struct_is_simple(GIStructInfo *info)
             case GI_TYPE_TAG_GHASH:
                 break;
             case GI_TYPE_TAG_INTERFACE:
-	    {
+            {
                 GIBaseInfo *interface = g_type_info_get_interface (type_info);
                 switch (g_base_info_get_type (interface)) {
-		case GI_INFO_TYPE_BOXED:
-		case GI_INFO_TYPE_STRUCT:
+                case GI_INFO_TYPE_BOXED:
+                case GI_INFO_TYPE_STRUCT:
                     if (!struct_is_simple ((GIStructInfo *)interface))
                         is_simple = FALSE;
                     break;
-		case GI_INFO_TYPE_UNION:
+                case GI_INFO_TYPE_UNION:
                     /* FIXME: Need to implement */
                     is_simple = FALSE;
                     break;
-		case GI_INFO_TYPE_ENUM:
-		case GI_INFO_TYPE_FLAGS:
+                case GI_INFO_TYPE_ENUM:
+                case GI_INFO_TYPE_FLAGS:
                     break;
-		case GI_INFO_TYPE_OBJECT:
-		case GI_INFO_TYPE_VFUNC:
-		case GI_INFO_TYPE_CALLBACK:
-		case GI_INFO_TYPE_INVALID:
-		case GI_INFO_TYPE_INTERFACE:
-		case GI_INFO_TYPE_FUNCTION:
-		case GI_INFO_TYPE_CONSTANT:
-		case GI_INFO_TYPE_ERROR_DOMAIN:
-		case GI_INFO_TYPE_VALUE:
-		case GI_INFO_TYPE_SIGNAL:
-		case GI_INFO_TYPE_PROPERTY:
-		case GI_INFO_TYPE_FIELD:
-		case GI_INFO_TYPE_ARG:
-		case GI_INFO_TYPE_TYPE:
-		case GI_INFO_TYPE_UNRESOLVED:
+                case GI_INFO_TYPE_OBJECT:
+                case GI_INFO_TYPE_VFUNC:
+                case GI_INFO_TYPE_CALLBACK:
+                case GI_INFO_TYPE_INVALID:
+                case GI_INFO_TYPE_INTERFACE:
+                case GI_INFO_TYPE_FUNCTION:
+                case GI_INFO_TYPE_CONSTANT:
+                case GI_INFO_TYPE_ERROR_DOMAIN:
+                case GI_INFO_TYPE_VALUE:
+                case GI_INFO_TYPE_SIGNAL:
+                case GI_INFO_TYPE_PROPERTY:
+                case GI_INFO_TYPE_FIELD:
+                case GI_INFO_TYPE_ARG:
+                case GI_INFO_TYPE_TYPE:
+                case GI_INFO_TYPE_UNRESOLVED:
                     is_simple = FALSE;
                     break;
-		}
+                }
 
                 g_base_info_unref (interface);
                 break;
-	    }
+            }
             }
         }
 
@@ -1196,7 +1187,7 @@ gjs_define_boxed_class(JSContext    *context,
     g_base_info_ref( (GIBaseInfo*) priv->info);
 
     gjs_debug(GJS_DEBUG_GBOXED, "Defined class %s prototype is %p class %p in object %p",
-              constructor_name, prototype, JS_GetClass(context, prototype), in_object);
+              constructor_name, prototype, JS_GET_CLASS(context, prototype), in_object);
 
     priv->can_allocate_directly = struct_is_simple (priv->info);
 
