@@ -36,6 +36,10 @@
 
 #include <girepository.h>
 
+static GType _array_type = G_TYPE_INVALID;
+static GType _byte_array_type = G_TYPE_INVALID;
+static GType _ptr_array_type = G_TYPE_INVALID;
+
 static JSBool gjs_value_from_g_value_internal(JSContext    *context,
                                               jsval        *value_p,
                                               const GValue *gvalue,
@@ -566,6 +570,20 @@ gjs_value_from_g_value_internal(JSContext    *context,
 
         obj = gjs_object_from_g_object(context, gobj);
         *value_p = OBJECT_TO_JSVAL(obj);
+    } else if (gtype == G_TYPE_STRV) {
+        if (!gjs_array_from_strv (context,
+                                  value_p,
+                                  g_value_get_boxed (gvalue))) {
+            gjs_throw(context, "Failed to convert strv to array");
+            return JS_FALSE;
+        }
+    } else if (g_type_is_a(gtype, G_TYPE_HASH_TABLE) ||
+               g_type_is_a(gtype, _array_type) ||
+               g_type_is_a(gtype, _byte_array_type) ||
+               g_type_is_a(gtype, _ptr_array_type)) {
+        gjs_throw(context,
+                  "Unable to introspect element-type of container in GValue");
+        return JS_FALSE;
     } else if (g_type_is_a(gtype, G_TYPE_BOXED)) {
         GjsBoxedCreationFlags boxed_flags;
         GIBaseInfo *info;
@@ -659,3 +677,11 @@ gjs_value_from_g_value(JSContext    *context,
 {
     return gjs_value_from_g_value_internal(context, value_p, gvalue, FALSE);
 }
+
+__attribute__((constructor)) void
+_gjs_value_init_types(void)
+{
+    _array_type = g_type_from_name("GArray");
+    _byte_array_type = g_type_from_name("GByteArray");
+    _ptr_array_type = g_type_from_name("GPtrArray");
+}
