@@ -37,7 +37,7 @@
 GQuark
 gjs_util_error_quark (void)
 {
-  return g_quark_from_static_string ("gjs-util-error-quark");
+    return g_quark_from_static_string ("gjs-util-error-quark");
 }
 
 typedef struct {
@@ -83,12 +83,12 @@ gjs_runtime_get_load_context(JSRuntime *runtime)
                                "is-load-context", TRUE,
                                NULL);
         gjs_runtime_set_data(runtime,
-                                "gjs-load-context",
-                                context,
-                                g_object_unref);
+                             "gjs-load-context",
+                             context,
+                             g_object_unref);
     }
 
-    return gjs_context_get_context(context);
+    return (JSContext*)gjs_context_get_native_context(context);
 }
 
 JSContext*
@@ -100,7 +100,7 @@ gjs_runtime_peek_load_context(JSRuntime *runtime)
     if (context == NULL) {
         return NULL;
     } else {
-        return gjs_context_get_context(context);
+        return (JSContext*)gjs_context_get_native_context(context);
     }
 }
 
@@ -109,9 +109,9 @@ gjs_runtime_clear_load_context(JSRuntime *runtime)
 {
     gjs_debug(GJS_DEBUG_CONTEXT, "Clearing load context");
     gjs_runtime_set_data(runtime,
-                            "gjs-load-context",
-                            NULL,
-                            NULL);
+                         "gjs-load-context",
+                         NULL,
+                         NULL);
     gjs_debug(GJS_DEBUG_CONTEXT, "Load context cleared");
 }
 
@@ -139,12 +139,12 @@ gjs_runtime_get_call_context(JSRuntime *runtime)
                                "runtime", runtime,
                                NULL);
         gjs_runtime_set_data(runtime,
-                                "gjs-call-context",
-                                context,
-                                g_object_unref);
+                             "gjs-call-context",
+                             context,
+                             g_object_unref);
     }
 
-    return gjs_context_get_context(context);
+    return (JSContext*)gjs_context_get_native_context(context);
 }
 
 static JSContext*
@@ -156,7 +156,7 @@ gjs_runtime_peek_call_context(JSRuntime *runtime)
     if (context == NULL) {
         return NULL;
     } else {
-        return gjs_context_get_context(context);
+        return (JSContext*)gjs_context_get_native_context(context);
     }
 }
 
@@ -165,9 +165,9 @@ gjs_runtime_clear_call_context(JSRuntime *runtime)
 {
     gjs_debug(GJS_DEBUG_CONTEXT, "Clearing call context");
     gjs_runtime_set_data(runtime,
-                            "gjs-call-context",
-                            NULL,
-                            NULL);
+                         "gjs-call-context",
+                         NULL,
+                         NULL);
     gjs_debug(GJS_DEBUG_CONTEXT, "Call context cleared");
 }
 
@@ -203,7 +203,7 @@ get_data_from_runtime(JSRuntime *runtime)
         rd = g_slice_new0(RuntimeData);
         rd->dynamic_classes = g_hash_table_new(g_direct_hash, g_direct_equal);
         gjs_runtime_set_data(runtime, "gjs-api-util-data",
-                                rd, runtime_data_destroy_notify);
+                             rd, runtime_data_destroy_notify);
     }
 
     return rd;
@@ -221,8 +221,8 @@ get_data_from_context(JSContext *context)
  */
 gboolean
 gjs_object_has_property(JSContext  *context,
-                           JSObject   *obj,
-                           const char *property_name)
+                        JSObject   *obj,
+                        const char *property_name)
 {
     return gjs_object_get_property(context, obj, property_name, NULL);
 }
@@ -235,13 +235,15 @@ gjs_object_has_property(JSContext  *context,
  */
 gboolean
 gjs_object_get_property(JSContext  *context,
-                           JSObject   *obj,
-                           const char *property_name,
-                           jsval      *value_p)
+                        JSObject   *obj,
+                        const char *property_name,
+                        jsval      *value_p)
 {
     jsval value;
     JSExceptionState *state;
 
+    JS_BeginRequest(context);
+
     value = JSVAL_VOID;
     state = JS_SaveExceptionState(context);
     JS_GetProperty(context, obj, property_name, &value);
@@ -250,6 +252,8 @@ gjs_object_get_property(JSContext  *context,
     if (value_p)
         *value_p = value;
 
+    JS_EndRequest(context);
+
     return value != JSVAL_VOID;
 }
 
@@ -269,6 +273,8 @@ gjs_object_require_property(JSContext       *context,
 {
     jsval value;
 
+    JS_BeginRequest(context);
+
     value = JSVAL_VOID;
     JS_GetProperty(context, obj, property_name, &value);
 
@@ -277,6 +283,7 @@ gjs_object_require_property(JSContext       *context,
 
     if (value != JSVAL_VOID) {
         JS_ClearPendingException(context); /* in case JS_GetProperty() was on crack */
+        JS_EndRequest(context);
         return TRUE;
     } else {
         /* remember gjs_throw() is a no-op if JS_GetProperty()
@@ -290,6 +297,8 @@ gjs_object_require_property(JSContext       *context,
             gjs_throw(context,
                       "No property '%s' in object %p (or its value was undefined)",
                       property_name, obj);
+
+        JS_EndRequest(context);
         return FALSE;
     }
 }
@@ -318,6 +327,8 @@ gjs_init_class_dynamic(JSContext      *context,
         return NULL;
     }
 
+    JS_BeginRequest(context);
+
     /* We replace the passed-in context and global object with our
      * runtime-global permanent load context. Otherwise, in a
      * process with multiple contexts, we'd arbitrarily define
@@ -325,6 +336,7 @@ gjs_init_class_dynamic(JSContext      *context,
      * class first, which is not desirable.
      */
     load_context = gjs_runtime_get_load_context(JS_GetRuntime(context));
+    JS_BeginRequest(load_context);
 
     /* JS_InitClass() wants to define the constructor in the global object, so
      * we give it a private and namespaced name... passing in the namespace
@@ -336,14 +348,14 @@ gjs_init_class_dynamic(JSContext      *context,
 
     prototype = NULL;
     if (gjs_object_get_property(load_context, JS_GetGlobalObject(load_context),
-                                   private_name, &value) &&
+                                private_name, &value) &&
         JSVAL_IS_OBJECT(value)) {
         jsval proto_val;
 
         g_free(private_name); /* don't need it anymore */
 
         if (!gjs_object_require_property(load_context, JSVAL_TO_OBJECT(value), NULL,
-                                            "prototype", &proto_val) ||
+                                         "prototype", &proto_val) ||
             !JSVAL_IS_OBJECT(proto_val)) {
             gjs_throw(load_context, "prototype was not defined or not an object?");
             goto error;
@@ -381,7 +393,7 @@ gjs_init_class_dynamic(JSContext      *context,
          * in_object
          */
         if (!gjs_object_require_property(load_context, JS_GetGlobalObject(load_context), NULL,
-                                            class_copy->base.name, &value))
+                                         class_copy->base.name, &value))
             goto error;
     }
     g_assert(value != JSVAL_VOID);
@@ -397,6 +409,8 @@ gjs_init_class_dynamic(JSContext      *context,
                            GJS_MODULE_PROP_FLAGS))
         goto error;
 
+    JS_EndRequest(load_context);
+    JS_EndRequest(context);
     return prototype;
 
  error:
@@ -407,18 +421,23 @@ gjs_init_class_dynamic(JSContext      *context,
         gjs_throw(context, "No exception was set, but class initialize failed somehow");
     }
 
+    JS_EndRequest(load_context);
+    JS_EndRequest(context);
     return NULL;
 }
 
 gboolean
 gjs_check_constructing(JSContext *context)
 {
+    JS_BeginRequest(context);
     if (!JS_IsConstructing(context)) {
+        JS_EndRequest(context);
         gjs_throw(context,
                   "Constructor called as normal method. Use 'new SomeObject()' not 'SomeObject()'");
         return FALSE;
     }
 
+    JS_EndRequest(context);
     return TRUE;
 }
 
@@ -430,13 +449,16 @@ gjs_get_instance_private_dynamic(JSContext      *context,
 {
     RuntimeData *rd;
     JSClass *obj_class;
+    void *instance;
 
     if (static_clasp->name != NULL) {
         g_warning("Dynamic class should not have a name in the JSClass struct");
         return NULL;
     }
 
-    obj_class = JS_GetClass(context, obj);
+    JS_BeginRequest(context);
+
+    obj_class = JS_GET_CLASS(context, obj);
     g_assert(obj_class != NULL);
 
     rd = get_data_from_context(context);
@@ -445,17 +467,22 @@ gjs_get_instance_private_dynamic(JSContext      *context,
     /* Check that it's safe to cast to DynamicJSClass */
     if (g_hash_table_lookup(rd->dynamic_classes, obj_class) == NULL) {
         gjs_throw(context,
-                     "Object %p proto %p doesn't have a dynamically-registered class, it has %s",
-                     obj, JS_GetPrototype(context, obj), obj_class->name);
+                  "Object %p proto %p doesn't have a dynamically-registered class, it has %s",
+                  obj, JS_GetPrototype(context, obj), obj_class->name);
+        JS_EndRequest(context);
         return NULL;
     }
 
     if (static_clasp != ((DynamicJSClass*) obj_class)->static_class) {
         gjs_throw(context, "Object is not a dynamically-registered class based on expected static class pointer");
+        JS_EndRequest(context);
         return NULL;
     }
 
-    return JS_GetInstancePrivate(context, obj, obj_class, argv);
+    instance = JS_GetInstancePrivate(context, obj, obj_class, argv);
+    JS_EndRequest(context);
+
+    return instance;
 }
 
 void*
@@ -466,13 +493,16 @@ gjs_get_instance_private_dynamic_with_typecheck(JSContext      *context,
 {
     RuntimeData *rd;
     JSClass *obj_class;
+    void *instance;
 
     if (static_clasp->name != NULL) {
         g_warning("Dynamic class should not have a name in the JSClass struct");
         return NULL;
     }
 
-    obj_class = JS_GetClass(context, obj);
+    JS_BeginRequest(context);
+
+    obj_class = JS_GET_CLASS(context, obj);
     g_assert(obj_class != NULL);
 
     rd = get_data_from_context(context);
@@ -480,14 +510,18 @@ gjs_get_instance_private_dynamic_with_typecheck(JSContext      *context,
 
     /* Check that it's safe to cast to DynamicJSClass */
     if (g_hash_table_lookup(rd->dynamic_classes, obj_class) == NULL) {
+        JS_EndRequest(context);
         return NULL;
     }
 
     if (static_clasp != ((DynamicJSClass*) obj_class)->static_class) {
+        JS_EndRequest(context);
         return NULL;
     }
 
-    return JS_GetInstancePrivate(context, obj, obj_class, argv);
+    instance = JS_GetInstancePrivate(context, obj, obj_class, argv);
+    JS_EndRequest(context);
+    return instance;
 }
 
 JSObject*
@@ -501,14 +535,17 @@ gjs_construct_object_dynamic(JSContext      *context,
     JSContext *load_context;
     JSObject *result;
 
+    JS_BeginRequest(context);
+
     /* We replace the passed-in context and global object with our
      * runtime-global permanent load context. Otherwise, JS_ConstructObject
      * can't find the constructor in whatever random global object is set
      * on the passed-in context.
      */
     load_context = gjs_runtime_get_load_context(JS_GetRuntime(context));
+    JS_BeginRequest(load_context);
 
-    proto_class = JS_GetClass(load_context, proto);
+    proto_class = JS_GET_CLASS(load_context, proto);
 
     rd = get_data_from_context(load_context);
 
@@ -530,6 +567,8 @@ gjs_construct_object_dynamic(JSContext      *context,
     if (!result)
         goto error;
 
+    JS_EndRequest(load_context);
+    JS_EndRequest(context);
     return result;
 
  error:
@@ -540,6 +579,8 @@ gjs_construct_object_dynamic(JSContext      *context,
         gjs_throw(context, "No exception was set, but object construction failed somehow");
     }
 
+    JS_EndRequest(load_context);
+    JS_EndRequest(context);
     return NULL;
 }
 
@@ -555,6 +596,8 @@ gjs_define_string_array(JSContext   *context,
     JSObject *array;
     int i;
 
+    JS_BeginRequest(context);
+
     if (!JS_EnterLocalRootScope(context))
         return JS_FALSE;
 
@@ -580,6 +623,8 @@ gjs_define_string_array(JSContext   *context,
     }
 
     JS_LeaveLocalRootScope(context);
+
+    JS_EndRequest(context);
     return array;
 }
 
@@ -588,6 +633,9 @@ gjs_value_debug_string(JSContext      *context,
                        jsval           value)
 {
     JSString *str;
+    const char *bytes;
+
+    JS_BeginRequest(context);
 
     str = JS_ValueToString(context, value);
 
@@ -598,7 +646,7 @@ gjs_value_debug_string(JSContext      *context,
              */
             JSClass *klass;
 
-            klass = JS_GetClass(context, JSVAL_TO_OBJECT(value));
+            klass = JS_GET_CLASS(context, JSVAL_TO_OBJECT(value));
             if (klass != NULL) {
                 str = JS_NewStringCopyZ(context, klass->name);
                 JS_ClearPendingException(context);
@@ -616,7 +664,11 @@ gjs_value_debug_string(JSContext      *context,
 
     g_assert(str != NULL);
 
-    return JS_GetStringBytes(str);
+    bytes = JS_GetStringBytes(str);
+
+    JS_EndRequest(context);
+
+    return bytes;
 }
 
 void
@@ -628,6 +680,8 @@ gjs_log_object_props(JSContext      *context,
     JSObject *props_iter;
     jsid prop_id;
 
+    JS_BeginRequest(context);
+
     /* We potentially create new strings, plus the property iterator,
      * that could get collected as we go through this process. So
      * create a local root scope.
@@ -672,6 +726,7 @@ gjs_log_object_props(JSContext      *context,
 
  done:
     JS_LeaveLocalRootScope(context);
+    JS_EndRequest(context);
 }
 
 void
@@ -691,6 +746,10 @@ gjs_explain_scope(JSContext  *context,
     load_context = gjs_runtime_peek_load_context(JS_GetRuntime(context));
     call_context = gjs_runtime_peek_call_context(JS_GetRuntime(context));
 
+    JS_BeginRequest(context);
+    JS_BeginRequest(load_context);
+    JS_BeginRequest(call_context);
+
     JS_EnterLocalRootScope(context);
 
     gjs_debug(GJS_DEBUG_SCOPE,
@@ -724,12 +783,18 @@ gjs_explain_scope(JSContext  *context,
     g_string_free(chain, TRUE);
 
     JS_LeaveLocalRootScope(context);
+
+    JS_EndRequest(call_context);
+    JS_EndRequest(load_context);
+    JS_EndRequest(context);
 }
 
 void
 gjs_log_exception_props(JSContext *context,
                         jsval      exc)
 {
+    JS_BeginRequest(context);
+
     /* This is useful when the exception was never sent to an error reporter
      * due to JSOPTION_DONT_REPORT_UNCAUGHT, or if the exception was not
      * a normal Error object so jsapi didn't know how to report it sensibly.
@@ -757,8 +822,8 @@ gjs_log_exception_props(JSContext *context,
         gjs_object_has_property(context, exc_obj, "message");
 
         gjs_log_object_props(context, exc_obj,
-                                GJS_DEBUG_ERROR,
-                                "  ");
+                             GJS_DEBUG_ERROR,
+                             "  ");
     } else if (JSVAL_IS_STRING(exc)) {
         gjs_debug(GJS_DEBUG_ERROR,
                   "Exception was a String");
@@ -766,6 +831,7 @@ gjs_log_exception_props(JSContext *context,
         gjs_debug(GJS_DEBUG_ERROR,
                   "Exception had some strange type");
     }
+    JS_EndRequest(context);
 }
 
 static JSBool
@@ -778,6 +844,8 @@ log_and_maybe_keep_exception(JSContext  *context,
     char *message;
     JSBool retval = JS_FALSE;
 
+    JS_BeginRequest(context);
+
     if (message_p)
         *message_p = NULL;
 
@@ -824,6 +892,8 @@ log_and_maybe_keep_exception(JSContext  *context,
  out:
     JS_RemoveRoot(context, &exc);
 
+    JS_EndRequest(context);
+
     return retval;
 }
 
@@ -850,39 +920,51 @@ try_to_chain_stack_trace(JSContext *src_context, JSContext *dst_context,
     jsval chained, src_stack, dst_stack, new_stack;
     JSString *new_stack_str;
 
+    JS_BeginRequest(src_context);
+    JS_BeginRequest(dst_context);
+
     if (!JSVAL_IS_OBJECT(src_exc))
-        return; // src_exc doesn't have a stack trace
+        goto out; // src_exc doesn't have a stack trace
 
     /* create a new exception in dst_context to get a stack trace */
     gjs_throw_literal(dst_context, "Chained exception");
     if (!(JS_GetPendingException(dst_context, &chained) &&
           JSVAL_IS_OBJECT(chained)))
-        return; // gjs_throw_literal didn't work?!
+        goto out; // gjs_throw_literal didn't work?!
     JS_ClearPendingException(dst_context);
 
     /* get stack trace for src_exc and chained */
     if (!(gjs_object_get_property(dst_context, JSVAL_TO_OBJECT(chained),
                                   "stack", &dst_stack) &&
           JSVAL_IS_STRING(dst_stack)))
-        return; // couldn't get chained stack
+        goto out; // couldn't get chained stack
     if (!(gjs_object_get_property(src_context, JSVAL_TO_OBJECT(src_exc),
                                   "stack", &src_stack) &&
           JSVAL_IS_STRING(src_stack)))
-        return; // couldn't get source stack
+        goto out; // couldn't get source stack
 
     /* add chained exception's stack trace to src_exc */
     new_stack_str = JS_ConcatStrings
         (dst_context, JSVAL_TO_STRING(src_stack), JSVAL_TO_STRING(dst_stack));
     if (new_stack_str==NULL)
-        return; // couldn't concatenate src and dst stacks?!
+        goto out; // couldn't concatenate src and dst stacks?!
     new_stack = STRING_TO_JSVAL(new_stack_str);
     JS_SetProperty(dst_context, JSVAL_TO_OBJECT(src_exc), "stack", &new_stack);
+
+ out:
+    JS_EndRequest(dst_context);
+    JS_EndRequest(src_context);
 }
 
 JSBool
 gjs_move_exception(JSContext      *src_context,
                    JSContext      *dest_context)
 {
+    JSBool success;
+
+    JS_BeginRequest(src_context);
+    JS_BeginRequest(dest_context);
+
     /* NOTE: src and dest could be the same. */
     jsval exc;
     if (JS_GetPendingException(src_context, &exc)) {
@@ -894,10 +976,15 @@ gjs_move_exception(JSContext      *src_context,
             JS_SetPendingException(dest_context, exc);
             JS_ClearPendingException(src_context);
         }
-        return JS_TRUE;
+        success = JS_TRUE;
     } else {
-        return JS_FALSE;
+        success = JS_FALSE;
     }
+
+    JS_EndRequest(dest_context);
+    JS_EndRequest(src_context);
+
+    return success;
 }
 
 JSBool
@@ -911,12 +998,17 @@ gjs_call_function_value(JSContext      *context,
     JSBool result;
     JSContext *call_context;
 
+    JS_BeginRequest(context);
+
     call_context = gjs_runtime_get_call_context(JS_GetRuntime(context));
+    JS_BeginRequest(call_context);
 
     result = JS_CallFunctionValue(call_context, obj, fval,
                                   argc, argv, rval);
     gjs_move_exception(call_context, context);
 
+    JS_EndRequest(call_context);
+    JS_EndRequest(context);
     return result;
 }
 
@@ -1014,6 +1106,9 @@ gjs_date_from_time_t (JSContext *context, time_t time)
     JSObject *date_constructor;
     jsval date_prototype;
     jsval args[1];
+    jsval result;
+
+    JS_BeginRequest(context);
 
     if (!JS_EnterLocalRootScope(context))
         return JSVAL_VOID;
@@ -1025,7 +1120,7 @@ gjs_date_from_time_t (JSContext *context, time_t time)
     if (!JS_GetProperty(context, date_constructor, "prototype", &date_prototype))
         gjs_fatal("Failed to get prototype from Date constructor");
 
-    date_class = JS_GetClass(context, JSVAL_TO_OBJECT (date_prototype));
+    date_class = JS_GET_CLASS(context, JSVAL_TO_OBJECT (date_prototype));
 
     if (!JS_NewNumberValue(context, ((double) time) * 1000, &(args[0])))
         gjs_fatal("Failed to convert time_t to number");
@@ -1033,8 +1128,11 @@ gjs_date_from_time_t (JSContext *context, time_t time)
     date = JS_ConstructObjectWithArguments(context, date_class,
                                            NULL, NULL, 1, args);
 
+    result = OBJECT_TO_JSVAL(date);
     JS_LeaveLocalRootScope(context);
-    return OBJECT_TO_JSVAL(date);
+    JS_EndRequest(context);
+
+    return result;
 }
 
 /**
@@ -1047,12 +1145,12 @@ gjs_date_from_time_t (JSContext *context, time_t time)
  * @Varargs: for each character in @format, a pair of a char * which is the name
  * of the argument, and a pointer to a location to store the value. The type of
  * value stored depends on the format character, as described below.
- * 
+ *
  * This function is inspired by Python's PyArg_ParseTuple for those
  * familiar with it.  It takes a format specifier which gives the
  * types of the expected arguments, and a list of argument names and
  * value location pairs.  The currently accepted format specifiers are:
- * 
+ *
  * s: A string, converted into UTF-8
  * z: Like 's', but may be null in JavaScript (which appears as NULL in C)
  * F: A string, converted into "filename encoding" (i.e. active locale)
@@ -1083,6 +1181,8 @@ gjs_parse_args (JSContext  *context,
     guint n_total;
     guint consumed_args;
 
+    JS_BeginRequest(context);
+
     va_start (args, argv);
 
     /* Check for optional argument specifier */
@@ -1112,7 +1212,7 @@ gjs_parse_args (JSContext  *context,
      * @i: The current integer position in fmt_args
      * @fmt_iter: A pointer to the character in fmt_args
      * @consumed_args: How many arguments we've taken from argv
-     * 
+     *
      * consumed_args can currently be different from 'i' because of the '|' character.
      */
     for (i = 0, consumed_args = 0, fmt_iter = format; *fmt_iter; fmt_iter++, i++) {
@@ -1135,86 +1235,100 @@ gjs_parse_args (JSContext  *context,
         g_return_val_if_fail (arg_location != NULL, JS_FALSE);
 
         js_value = argv[consumed_args];
-        
+
         switch (*fmt_iter) {
-            case 'o': {
-               if (!JSVAL_IS_OBJECT(js_value)) {
-                   arg_error_message = "Not an object";
-               } else {
-                   JSObject **arg = arg_location;
-                   *arg = JSVAL_TO_OBJECT(js_value);
-               }
+        case 'o': {
+            if (!JSVAL_IS_OBJECT(js_value)) {
+                arg_error_message = "Not an object";
+            } else {
+                JSObject **arg = arg_location;
+                *arg = JSVAL_TO_OBJECT(js_value);
             }
+        }
             break;
-            case 's':
-            case 'z': {
-                char **arg = arg_location;
-                
-                if (*fmt_iter == 'z' && JSVAL_IS_NULL(js_value)) {
-                    *arg = NULL;
-                } else {
-                    if (gjs_try_string_to_utf8 (context, js_value, arg, &arg_error)) {
-                        unwind_strings[n_unwind++] = *arg;
-                        g_assert(n_unwind < MAX_UNWIND_STRINGS);
-                    }
-                }    
-            }
-            break;
-            case 'F': {
-                char **arg = arg_location;
-                
-                if (gjs_try_string_to_filename (context, js_value, arg, &arg_error)) {
+        case 's':
+        case 'z': {
+            char **arg = arg_location;
+
+            if (*fmt_iter == 'z' && JSVAL_IS_NULL(js_value)) {
+                *arg = NULL;
+            } else {
+                if (gjs_try_string_to_utf8 (context, js_value, arg, &arg_error)) {
                     unwind_strings[n_unwind++] = *arg;
                     g_assert(n_unwind < MAX_UNWIND_STRINGS);
                 }
             }
-            break;
-            case 'i': {
-                if (!JS_ValueToInt32(context, js_value, (gint32*) arg_location)) {
-                    /* Our error message is going to be more useful */
-                    JS_ClearPendingException(context);
-                    arg_error_message = "Couldn't convert to integer";
-                }
-            }
-            break;
-            case 'u': {
-                gdouble num;
-                if (!JS_ValueToNumber(context, js_value, &num)) {
-                    /* Our error message is going to be more useful */
-                    JS_ClearPendingException(context);
-                    arg_error_message = "Couldn't convert to unsigned integer";
-                } else if (num > G_MAXUINT32 || num < 0) {
-                    arg_error_message = "Value is out of range";
-                } else {
-                    *((guint32*) arg_location) = num;
-                }
-            }
-            break;
-            default:
-              g_assert_not_reached ();
         }
-        
+            break;
+        case 'F': {
+            char **arg = arg_location;
+
+            if (gjs_try_string_to_filename (context, js_value, arg, &arg_error)) {
+                unwind_strings[n_unwind++] = *arg;
+                g_assert(n_unwind < MAX_UNWIND_STRINGS);
+            }
+        }
+            break;
+        case 'i': {
+            if (!JS_ValueToInt32(context, js_value, (gint32*) arg_location)) {
+                /* Our error message is going to be more useful */
+                JS_ClearPendingException(context);
+                arg_error_message = "Couldn't convert to integer";
+            }
+        }
+            break;
+        case 'u': {
+            gdouble num;
+            if (!JS_ValueToNumber(context, js_value, &num)) {
+                /* Our error message is going to be more useful */
+                JS_ClearPendingException(context);
+                arg_error_message = "Couldn't convert to unsigned integer";
+            } else if (num > G_MAXUINT32 || num < 0) {
+                arg_error_message = "Value is out of range";
+            } else {
+                *((guint32*) arg_location) = num;
+            }
+        }
+            break;
+        case 'f': {
+            double num;
+            if (!JS_ValueToNumber(context, js_value, &num)) {
+                /* Our error message is going to be more useful */
+                JS_ClearPendingException(context);
+                arg_error_message = "Couldn't convert to double";
+            } else {
+                *((double*) arg_location) = num;
+            }
+        }
+            break;
+        default:
+            g_assert_not_reached ();
+        }
+
         if (arg_error != NULL)
             arg_error_message = arg_error->message;
-        
+
         if (arg_error_message != NULL) {
-            gjs_throw(context, "Error invoking %s, at argument %d (%s): %s", function_name, 
+            gjs_throw(context, "Error invoking %s, at argument %d (%s): %s", function_name,
                       consumed_args+1, argname, arg_error_message);
             g_clear_error (&arg_error);
             goto error_unwind;
         }
-        
+
         consumed_args++;
     }
 
     va_end (args);
+
+    JS_EndRequest(context);
     return JS_TRUE;
 
-  error_unwind:
+ error_unwind:
     va_end (args);
     /* We still own the strings in the error case, free any we converted */
     for (i = 0; i < n_unwind; i++) {
         g_free (unwind_strings[i]);
     }
+    JS_EndRequest(context);
     return JS_FALSE;
 }
