@@ -43,8 +43,9 @@ JSBool gjs_js_one_value_to_dbus   (JSContext          *context,
                                    jsval               value,
                                    DBusMessageIter    *iter,
                                    DBusSignatureIter  *sig_iter);
-                                            
-void gjs_js_set_current_message   (DBusMessage        *message);
+
+void gjs_js_push_current_message  (DBusMessage        *message);
+void gjs_js_pop_current_message   ();
 
 
 G_END_DECLS
