@@ -6,6 +6,7 @@ if (!('assertEquals' in this)) { /* allow running this test standalone */
 
 // We use Gio to have some objects that we know exist
 const Gio = imports.gi.Gio;
+const Lang = imports.lang;
 
 const INT8_MIN = (-128);
 const INT16_MIN = (-32767-1);
@@ -45,10 +46,10 @@ function testLifeUniverseAndEverything() {
     assertEquals(-42, Everything.test_int64(-42));
 
     assertEquals(42, Everything.test_uint64(42));
-    
+
     assertEquals(42, Everything.test_short(42));
     assertEquals(-42, Everything.test_short(-42));
-    
+
     assertEquals(42, Everything.test_ushort(42));
 
     assertEquals(42, Everything.test_int(42));
@@ -72,13 +73,9 @@ function testLifeUniverseAndEverything() {
     assertEquals(42, Everything.test_double(42));
     assertEquals(-42, Everything.test_double(-42));
 
-    let now = new Date();
-    let bounced = Everything.test_timet(now);
-    assertEquals(now.getFullYear(), bounced.getFullYear());
-    assertEquals(now.getMonth(), bounced.getMonth());
-    assertEquals(now.getDay(), bounced.getDay());
-    assertEquals(now.getHours(), bounced.getHours());
-    assertEquals(now.getSeconds(), bounced.getSeconds());
+    let now = Math.floor(new Date().getTime() / 1000);
+    let bounced = Math.floor(Everything.test_timet(now));
+    assertEquals(bounced, now);
 }
 
 function testLimits() {
@@ -115,9 +112,9 @@ function testNoImplicitConversionToUnsigned() {
 
 function testBadConstructor() {
     try {
-	Gio.AppLaunchContext();
+        Gio.AppLaunchContext();
     } catch (e) {
-	assert(e.message.indexOf("Constructor called as normal method") >= 0);
+        assert(e.message.indexOf("Constructor called as normal method") >= 0);
     }
 }
 
@@ -141,6 +138,13 @@ function testStrv() {
     assertEquals("3", strv[2]);
 }
 
+function testInAfterOut() {
+    const str = "hello";
+
+    let len = Everything.test_int_out_utf8(str);
+    assertEquals("testInAfterOut", str.length, len);
+}
+
 function testUtf8() {
     const CONST_STR = "const \u2665 utf8";
     const NONCONST_STR = "nonconst \u2665 utf8";
@@ -151,6 +155,9 @@ function testUtf8() {
     Everything.test_utf8_const_in(CONST_STR);
     assertEquals(NONCONST_STR, Everything.test_utf8_out());
     assertEquals(NONCONST_STR, Everything.test_utf8_inout(CONST_STR));
+    assertEquals(NONCONST_STR, Everything.test_utf8_inout(CONST_STR));
+    assertEquals(NONCONST_STR, Everything.test_utf8_inout(CONST_STR));
+    assertEquals(NONCONST_STR, Everything.test_utf8_inout(CONST_STR));
 }
 
 function testFilenameReturn() {
@@ -166,22 +173,29 @@ function testStaticMeth() {
 }
 
 function testClosure() {
+    let arguments_length = -1;
     let someCallback = function() {
+                           arguments_length = arguments.length;
                            return 42;
                        };
 
     let i = Everything.test_closure(someCallback);
 
+    assertEquals('callback arguments length', 0, arguments_length);
     assertEquals('callback return value', 42, i);
 }
 
 function testClosureOneArg() {
+    let arguments_length = -1;
     let someCallback = function(someValue) {
+                           arguments_length = arguments.length;
+                           assertEquals(1, arguments.length);
                            return someValue;
                        };
 
     let i = Everything.test_closure_one_arg(someCallback, 42);
 
+    assertEquals('callback arguments length', 1, arguments_length);
     assertEquals('callback with one arg return value', 42, i);
 }
 
@@ -192,27 +206,23 @@ function testCallback() {
     assertEquals('Callback', Everything.test_callback(callback), 42);
 
     assertEquals('CallbackNull', Everything.test_callback(null), 0);
-    assertEquals('CallbackUndefined', Everything.test_callback(undefined), 0);
-}
-
-function testCallbackUserData() {
-    let callbackUserData = function(userData) {
-                               return userData.foo.length;
-                           };
-
-    let userData = {'foo': 'bar'};
-    assertEquals('CallbackUserData', Everything.test_callback_user_data(
-                                     callbackUserData, userData), userData.foo.length);
+    assertRaises('CallbackUndefined', function () { Everything.test_callback(undefined) });
 }
 
 function testCallbackDestroyNotify() {
-    let called = 0;
-    let test = function(userData) {
-                   called++;
-                   return userData;
-               };
-    assertEquals('CallbackDestroyNotify', Everything.test_callback_destroy_notify(test, 42), 42);
-    assertEquals('CallbackDestroyNotify', called, 1);
+    let testObj = {
+        called: 0,
+        test: function(data) {
+            this.called++;
+            return data;
+        }
+    };
+    assertEquals('CallbackDestroyNotify',
+                 Everything.test_callback_destroy_notify(Lang.bind(testObj,
+                     function() {
+                         return testObj.test(42);
+                     })), 42);
+    assertEquals('CallbackDestroyNotify', testObj.called, 1);
     assertEquals('CallbackDestroyNotify', Everything.test_callback_thaw_notifications(), 42);
 }
 
@@ -289,17 +299,17 @@ function testArrayOut() {
 // Convert an object to a predictable (not-hash-order-dependent) string
 function objToString(v) {
     if (typeof(v) == "object") {
-	let keys = [];
-	for (let k in v)
-	    keys.push(k);
-	keys.sort();
-	return "{" + keys.map(function(k) {
-	    return k + ":" + objToString(v[k]);
-	}) + "}";
+        let keys = [];
+        for (let k in v)
+            keys.push(k);
+        keys.sort();
+        return "{" + keys.map(function(k) {
+            return k + ":" + objToString(v[k]);
+        }) + "}";
     } else if (typeof(v) == "string") {
-	return '"' + v + '"';
+        return '"' + v + '"';
     } else {
-	return v;
+        return v;
     }
 }
 
@@ -375,4 +385,64 @@ function testSignalWithStaticScopeArg() {
     assertEquals('signal handler was passed arg as reference', 44, b.some_int);
 }
 
+function testTortureSignature0() {
+    let [y, z, q] = Everything.test_torture_signature_0(42, 'foo', 7);
+    assertEquals(Math.floor(y), 42);
+    assertEquals(z, 84);
+    assertEquals(q, 10);
+}
+
+function testTortureSignature1Fail() {
+    assertRaises(function () {
+        let [success, y, z, q] = Everything.test_torture_signature_1(42, 'foo', 7);
+    });
+}
+
+function testTortureSignature1Success() {
+    let [success, y, z, q] = Everything.test_torture_signature_1(11, 'barbaz', 8);
+    assertEquals(Math.floor(y), 11);
+    assertEquals(z, 22);
+    assertEquals(q, 14);
+}
+
+function testTortureSignature2() {
+    let [y, z, q] = Everything.test_torture_signature_2(42, function () {
+        }, 'foo', 7);
+    assertEquals(Math.floor(y), 42);
+    assertEquals(z, 84);
+    assertEquals(q, 10);
+}
+
+function testObjTortureSignature0() {
+    let o = new Everything.TestObj();
+    let [y, z, q] = o.torture_signature_0(42, 'foo', 7);
+    assertEquals(Math.floor(y), 42);
+    assertEquals(z, 84);
+    assertEquals(q, 10);
+}
+
+function testObjTortureSignature1Fail() {
+    let o = new Everything.TestObj();
+    assertRaises(function () {
+        let [success, y, z, q] = o.torture_signature_1(42, 'foo', 7);
+    });
+}
+
+function testObjTortureSignature1Success() {
+    let o = new Everything.TestObj();
+    let [success, y, z, q] = o.torture_signature_1(11, 'barbaz', 8);
+    assertEquals(Math.floor(y), 11);
+    assertEquals(z, 22);
+    assertEquals(q, 14);
+}
+
+function testStrvInGValue() {
+    let v = Everything.test_strv_in_gvalue();
+
+    assertEquals(v.length, 3);
+    assertEquals(v[0], "one");
+    assertEquals(v[1], "two");
+    assertEquals(v[2], "three");
+}
+
 gjstestRun();
